package CoreSim::ALT;

use strict;
use CoreSim::ALT::ALTHTopology;
use CoreSim::ALT::ALTHMetrics;
use CoreSim::Utilities::IPUtilities;

sub new(){
	my $invocant = shift;
	my ($topology,$startAggregationLevel, $aggregationIndex,$outputFilePath, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	#for now ALT is ONLY iterative
	my $resolverType = 1;
	
	my $self={
		"topology" => $topology,
		"startAggregationLevel" => $startAggregationLevel,
		"aggregationIndex" => $aggregationIndex,
		"resolverType" => $resolverType,
		"outputFilePath" => $outputFilePath,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
	
	&initializeAll($self);
	return $self;
}

sub initializeAll(){
	my $self=shift;
	
	my $topology = $self->{"topology"};
	my $startAggregationLevel = $self->{"startAggregationLevel"};
	my $aggregationIndex = $self->{"aggregationIndex"};
	my $debug = $self->{"_DEBUG"};
	my $ipUtility = CoreSim::Utilities::IPUtilities->new();
	my $outputFilePath = $self->{"outputFilePath"};
	
	my $altHTopology = CoreSim::ALT::ALTHTopology->new($topology, $startAggregationLevel, $aggregationIndex,$ipUtility, $debug);
	my $altHMetrics = CoreSim::ALT::ALTHMetrics->new($topology, $altHTopology,$ipUtility, $aggregationIndex,$outputFilePath, $debug);
	
	$self->{"altHTopology"} = $altHTopology;
	$self->{"altHMetrics"} = $altHMetrics;
}

sub getALTLookupDelayFromALTRouter(){
	my $self= shift;
	my ($altRouterPrefix, $searchedIP) = @_;
	
	return $self->{"altHMetrics"}->getALTLookupDelayFromALTRouter($altRouterPrefix, $searchedIP);
}

# It just calls the getALTLookupDelayFromALTRouter. It is used because I need a common name for both lookup functions (chord, alt) and 
# I didn't want to change the name of the above function

sub getLookupDelay(){
	my $self= shift;
	my ($altRouterPrefix, $searchedIP) = @_;
	
	return &getALTLookupDelayFromALTRouter($self,$altRouterPrefix, $searchedIP);
}

#returns the resolver type for ALT (for now ALT is ONLY iterative)
sub getResolverType(){
	my $self=shift;
	
	return $self->{"resolverType"};
}

#each mapping system has this function and it uses it to transform a prefix (given as parameter) into an appropriate form
#to be used by the getLookupDelayFunction. Here the prefix is not transformed, it remains the same(ALT uses a prefix to identify the
#ITR node)
sub getAssociatedNodeForPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	return $prefix;
}

sub setNodeIP(){
	my $self=shift;
	my ($altNodeID, $ip) = @_;
	
	my $altNode = $self->{"altHTopology"}->getRouterWithPrefix($altNodeID);
	$altNode->setPOPIP($ip);	
}

#return 1
1