package CoreSim::ALT::ALTHTopology;

use strict;
use CoreSim::ALT::ALTRouter;
#use CoreSim::Utilities::IPUtilities;

sub new(){
	my $invocant = shift;
	my ($topology, $startAggregationLevel, $aggregationIndex,$ipUtility, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	
	#this is the root node
	my $root = CoreSim::ALT::ALTRouter->new(undef,$ipUtility, $_DEBUG);
	$root->setPrefix("0.0.0.0/0");
	
	my $altRoutersHash={};
	
	my $self={
		"topology" => $topology,
		"startAggregationLevel" => $startAggregationLevel,
		"aggregationIndex" => $aggregationIndex,
		"altRoutersHash" => $altRoutersHash,
		"ipUtility" => $ipUtility,
		"startAggregationLevel"=> $startAggregationLevel,
		"root" => $root,
		"_DEBUG" => $_DEBUG,
	};
	
#	$self->{"_DEBUG"} = 3;
	bless($self, $class);
	
	&createALTTopology($self);
	return $self;
}

sub createALTTopology(){
	my $self= shift;
	
	my $topology = $self->{"topology"};
	
	printf("Creating the ALT Topology\n") if ($self->{"_DEBUG"});
	
	my $startAggregationLevel = $self->{"startAggregationLevel"};
	my $aggregationIndex = $self->{"aggregationIndex"};
	
	#hardcoded for now
	my $nbOfLevels =2;
	
	&createMixedTopology($self, $startAggregationLevel, $nbOfLevels, $aggregationIndex);
}

sub createMixedTopology(){
	my ($self,$startAggregationLevel, $nbOfLevels, $aggregationIndex)= @_;
		
	#create the 2 layers of aggregators
	&createALTAggregators($self, $startAggregationLevel, $nbOfLevels, $aggregationIndex);
	
	#add all the routers responsible for the prefixes to the ALT Topology
	&createALTRouters($self);
	
	#show the virtual routers if the debug level is higher than 2
	&showVirtualRouters($self) if ($self->{"_DEBUG"}>=2);
	
}

sub createALTAggregators(){
	my ($self, $startAggregationLevel, $nbOfLevels, $aggregationIndex)=@_;
	
	my $topology = $self->{"topology"};
	my $debug = $self->{"_DEBUG"};
	my $ipUtility = $self->{"ipUtility"};
	my $root = $self->{"root"};
	my $altRoutersHash = $self->{"altRoutersHash"};

	my $previousRouterLevel;
	my $currentRouterLevel;
	my $currentAggregation=0;
	
	printf("Creating the Aggregators!\n") if ($self->{"_DEBUG"});
	
	#create the aggregators
	#for all the levels
	
	$currentAggregation = $startAggregationLevel;
	for (my $i=0; $i< $nbOfLevels; $i++){
		
		#the old currentRouterLevel becomes the previousRouterLevel
		$previousRouterLevel = $currentRouterLevel;
		
		printf("\n The current aggregation level %s\n", $currentAggregation) if ($self->{"_DEBUG"});

		if (defined($previousRouterLevel)){
			
			my $count = 0;
			$currentRouterLevel = ();
			
			#for all the previously introduced routers
			foreach my $element ( @{$previousRouterLevel} ){
				
				#increment their prefix value with the value determined by the currentAggregationLevel
				for (my $j = 0; $j<(2**$aggregationIndex) ; $j++){
					
					my $value = $element->{"value"} + ( $j<<(32-$currentAggregation) );
						
					#compute the router prefix
					my $tmpIP = $ipUtility->numberToIP($value);
					my $tmpMask = $currentAggregation;
					my $tmpPrefix = "$tmpIP/$tmpMask";
					
					
					#do not add prefixes of the form 0.* to the alt topology 
					if ( (!&firstEightBitsZero($tmpIP)) && (!&isDorEIPClass($tmpIP)) ){
						
						my $parentALTRouter = $element->{"node"};
						
						#compute router altLevel
						my $altLevel = $parentALTRouter->getALTLevel() + 1;
						
						#create the element
						my $altRouter = CoreSim::ALT::ALTRouter->new($element->{"node"}, $ipUtility, $altLevel, $debug);
						$altRouter->setPrefix($tmpPrefix);
						
						#connect the parent to the child
						$parentALTRouter->addChild($altRouter);
						
						#add the router to the altRouterHash (we can find the routers by their prefix)
						$altRoutersHash->{$tmpPrefix} = $altRouter;
						
						#adding the node to the routerLevel
						my %tmpHash;
						$tmpHash{"node"} = $altRouter;
						$tmpHash{"value"} = $value;
						$currentRouterLevel->[$count]=\%tmpHash;
						$count++;
						
						printf("Adding prefix %s with the parent %s\n", $tmpPrefix,$parentALTRouter->getPrefix())
								if ($self->{"_DEBUG"}>=3);
						
					}
				}
			}
		

		
			
		#these are the routers in the first level
		}else{
			my $altLevel = 1;
			
			for (my $j =0; $j<(2**$startAggregationLevel); $j++){
				my $value = $j<<(32-$currentAggregation);
				
				#create the element
				my $altRouter = CoreSim::ALT::ALTRouter->new($root, $ipUtility, $altLevel, $debug);
				
				#set the router prefix
				my $tmpIP = $ipUtility->numberToIP($value);
				my $tmpMask = $currentAggregation;
				my $tmpPrefix = "$tmpIP/$tmpMask";
				
				$altRouter->setPrefix($tmpPrefix);
				$root->addChild($altRouter);
				
				#add the router to the altRouterHash (we can find the routers by their prefix)
				$altRoutersHash->{$tmpPrefix} = $altRouter;
				
				#create the level  (the root servers level)
				my %tmpHash;
				$tmpHash{"node"} = $altRouter;
				$tmpHash{"value"} = $value;
				$currentRouterLevel->[$j]=\%tmpHash;
				
				printf("Adding prefix %s on the root layer\n", $tmpPrefix) if ($self->{"_DEBUG"}>=3);
				
			}
			
			#set the delays within the root layer (I call root layer the first layer "under" tree rootNode)
			#on this layer we can find the root ALT routers
			&createRootLayerMesh($self, $currentRouterLevel);
			&associatePOPsToRootRouters($self, $currentRouterLevel);
		}
		
		#increment the aggregation with the aggregationIndex
		$currentAggregation += $aggregationIndex; 
	}
	
	$self->{"altRoutersHash"} = $altRoutersHash;
}

sub createALTRouters(){
	my $self = shift;
	
	my $topology = $self->{"topology"};
	
	#we use the prefix list (not the filtered one)
	my $prefixList = $topology->getFilteredPrefixList();
	my $ipUtility = $self->{"ipUtility"};
	my $debug = $self->{"_DEBUG"}; 
	my $altRoutersHash = $self->{"altRoutersHash"};
	
	my $root = $self->{"root"};
	
	# $i will take all the possible values for the prefix masks. From the prefixFile I noticed that there are no prefixes more general 
	#than /8 and they are as specific as /32
	my $prefixesWithParents = 0;
	for (my $i=8; $i<= 32; $i++){
		
		printf("Adding altRouters responsible for prefixes \/%s\n", $i);
		
		foreach my $prefix (@$prefixList){
			
			if ($ipUtility->getMaskFromPrefix($prefix) == $i){
				
				#this happends if the prefix is among those in the first layer (the relations among them are already settled within
				#the createALTAggregators())
				if ( defined($altRoutersHash->{$prefix}) ){
					my $altRouter = $altRoutersHash->{$prefix};
					
					#set pop specific information for the altRouter
					my $pop = $topology->getBorderPOPForPrefix($prefix);
					my $popIP = $topology->getIPForPOP($pop);
					my $as = $topology->getASForPOP($pop);
					
					$altRouter->setPOPNumber($pop);
					$altRouter->setPOPIP($popIP);	
					$altRouter->setAS($as);
					
					printf("Adding POP information for router responsible for prefix %s\n", $prefix) if ($self->{"_DEBUG"}>=3);
				#otherwise, if the altRouter added is new to the topology search for its parent
				}else{
					
					#search for the parent router (the min prefix is 8)
					my $parentALTRouter = &findParentNodeForPrefix($self, $prefix, 8, $altRoutersHash);
					
					#compute router altLevel
					my $altLevel = $parentALTRouter->getALTLevel() + 1 ;
					
					#create alt router
					my $altRouter = CoreSim::ALT::ALTRouter->new($parentALTRouter, $ipUtility, $altLevel, $debug);
					$altRouter->setPrefix($prefix);
					
					#add the router to the altRoutersHash
					$altRoutersHash->{$prefix} = $altRouter;
					
					#set the altRouter as a child for the parent	
					$parentALTRouter->addChild($altRouter);
					
					#set pop specific information for the altRouter
					my $pop = $topology->getBorderPOPForPrefix($prefix);
					my $popIP = $topology->getIPForPOP($pop);
					my $as = $topology->getASForPOP($pop);
					
					$altRouter->setPOPNumber($pop);
					$altRouter->setPOPIP($popIP);	
					$altRouter->setAS($as);
					
					#if the parent is a virtualPOP (it has no IP associated to it) then the first child
					#will "lend" its pop specific information to the parent
					if ( &virtualPOP($parentALTRouter)){
						&setChildAttributesToParent($parentALTRouter, $altRouter);
					}
					
					printf("Adding router responsible for prefix %s with parent %s\n", $prefix, $parentALTRouter->getPrefix()) 
							if ($self->{"_DEBUG"}>=3);
					
				}	
			}
			
		}
		
	}
	
}

sub findParentNodeForPrefix(){
	my $self=shift;
	my ($childPrefix, $minMask, $altRoutersHash) = @_;
	
	my $ipUtility = $self->{"ipUtility"};

	my $endLoop =0;
	my $possibleParentPrefix;
	
	my $mask = $ipUtility->getMaskFromPrefix($childPrefix);
	my $prefix = $childPrefix;
	
	#search the hierarchy for a parent prefix
	while (($mask > $minMask) && (!$endLoop)){
		
		#get a possible parent prefix with a mask decreased by 1
		$possibleParentPrefix = $ipUtility->computeParentPrefix($prefix, 1);
		
		#if the parent does exist stop searching
		if (defined($altRoutersHash->{$possibleParentPrefix})){
			$endLoop = 1;
		}
		
		$mask -= 1;
		$prefix = $possibleParentPrefix;
	}
	
#	printf("prefix = %s possible parent prefix = %s endLoop = %s\n",$childPrefix, $possibleParentPrefix, $endLoop );
	#if the parent is not found then root should be the parent?
	if ($endLoop==0){
		printf("(EE) There is no parent for the prefix %s!\n", $childPrefix);
		return $self->{"root"};
	}else{
		return $altRoutersHash->{$possibleParentPrefix};
	}
	
}

sub firstEightBitsZero(){
	
	my $ip = shift;
	
	my @result = split(/\./, $ip);
	if ($result[0] == 0 ){
		return 1;
	}else{
		return 0;
	}
}

sub isDorEIPClass(){
	my $ip = shift;
	
	my @result = split(/\./, $ip);
	if ($result[0] >= 224){
		return 1;
	}else{
		return 0;
	}
}

sub virtualPOP(){
	my ($parentRouter) = @_;
	
	#if the parent has no IP that means that it is "virtual"
	if ( ! defined( $parentRouter->getPOPIP() )){
		return 1;
	}else{
		return 0;
	}
}

sub setChildAttributesToParent(){
	my ($parentALTRouter, $altRouter) = @_;	
	
	$parentALTRouter->setPOPNumber($altRouter->getPOPNumber());
	$parentALTRouter->setPOPIP($altRouter->getPOPIP());
	$parentALTRouter->setAS($altRouter->getAS());
	$parentALTRouter->setEndorsedByChild();
}

sub associatePOPsToRootRouters(){
	my ($self, $routerLevel) = @_;
	
	my $topology = $self->{"topology"};
	my $numberOfIPsPerRootRouter = 2;
	my $numberOfRootRouters = 8;
	
	my $numberOfRequiredPOPs = $numberOfIPsPerRootRouter * $numberOfRootRouters;
	
	my $requestedPOPs = $topology->getBestConnectedNPOPs($numberOfRequiredPOPs);
	
	for (my $i=0;$i< $numberOfRequiredPOPs; $i+= $numberOfIPsPerRootRouter ){
		for (my $j=0; $j< $numberOfIPsPerRootRouter; $j++){
			
			my $routerNumber = $i/$numberOfIPsPerRootRouter;
			
			my $routerInfo = $requestedPOPs->[$i+$j];
			$routerLevel->[$routerNumber]->{"node"}->addRootRouterInfo($routerInfo);
		}
	}
	
}

sub createRootLayerMesh(){
	my ($self, $routerLevel) = @_;
	
	my $nbOfRouters = scalar(@$routerLevel);
	
	for (my $i=0;$i<$nbOfRouters ; $i++){
		for (my $j=0; $j<$nbOfRouters; $j++){
			if ($i!=$j){
				my $router1 = $routerLevel->[$i]->{"node"};
				my $router2 = $routerLevel->[$j]->{"node"};
				
				$router1->addFIBEntry($router2->getPrefix, $router2);
			}
			
		}
	}
}

sub getALTRouterForPrefix(){
	my $self = shift;
	my $prefix = shift;
	
	return $self->{"altRoutersHash"}->{$prefix};	
}

sub getRoot(){
	my $self= shift;
	
	return $self->{"root"};
}

sub showVirtualRouters(){
	my $self=shift;
	
	my $altRoutersHash = $self->{"altRoutersHash"};
	
	foreach my $altRouterPrefix (keys %$altRoutersHash){
		
		my $altRouter = $altRoutersHash->{$altRouterPrefix};
		if (&virtualPOP($altRouter) && !$altRouter->isRootRouter()){
			printf("The alt router with the prefix %s is a virtual one\n", $altRouter->getPrefix());
		}
	}
}

sub getRouterWithPrefix(){
	my $self=shift;
	my ($nodePrefix) = @_;
	
	my $altRoutersHash = $self->{"altRoutersHash"};
	
	return $altRoutersHash->{$nodePrefix};	
}

#return 1 at the end
1