package CoreSim::Chord::ChordNode;

use strict;

#constructor for the class
sub new(){
	my $invocant = shift;
	my ($chordID,$ip,$popNumber,$as,$successor,$predecessor,$keys, $fingerTable) = @_;
	
	my $class = ref ($invocant) || $invocant;
	
	my $self = {};
	
	#localization information. This shouldn't change as our topology is static.
	$self->{"chordID"} = $chordID;
	$self->{"ip"} = $ip;
	$self->{"popNumber"} = $popNumber; 
	$self->{"as"} = $as;
	
	#both successor and predecessor are references to ChordNode objects. These are subject to changes thus setter methods exist.
	$self->{"successor"} = $successor;
	$self->{"predecessor"} = $predecessor;
	$self->{"keys"} = $keys;
	
	#add the fingerTable 
	$self->{"fingerTable"} = $fingerTable;

	bless($self, $class);
	
	return $self;
}

sub getSuccessor(){
	my $self = shift;
	return $self->{"successor"};
}

sub getPredecessor(){
	my $self = shift;
	return $self->{"predecessor"};	
}

sub getKeys(){
	my $self = shift;
	return $self->{"keys"};
}

sub getChordID(){
	my $self = shift;
	return $self->{"chordID"};
}

#this is temporary (not exactly the proper way to implement this!)
sub getPrefix(){
	my $self=shift;
	
	return $self->{"keys"};
}

sub getFingerTable(){
	my $self = shift;
	return $self->{"fingerTable"};
}

sub getPOP(){
	my $self = shift;
	return $self->{"popNumber"};	
}

sub getIP(){
	my $self=shift;
	
	return $self->{"ip"};	
}

sub getAS(){
	my $self=shift;
	
	return $self->{"as"};	
}

#returns a reference to a hash that contains all the keys owned by the node and smaller than the given limit
sub getKeysBeforeKey(){
	my $self = shift;
	my $keyLimit = shift;
	my $currentKeys = $self->{"keys"};
	
	my @keyList = keys %{$currentKeys};
	my $returnHash;
	
	foreach my $key (@keyList){
		if( $key<= $keyLimit){
			$returnHash->{$key} = 1;
		}
	}
	
	return $returnHash;
}

#deletes the keys with the ID smaller than keyLimit(The limit is in fact the new node introduced in the Chord Circle)
sub deleteKeysBeforeKey(){
	my $self = shift;
	my $keyLimit = shift;
	my $currentKeys = $self->{"keys"};
	
	my @keyList = keys %{$currentKeys};
	
	#delete the keys with smaller ID than the keyLimit. There might be a faster way to do this in perl.. i just don't know it(n00b)
	foreach my $key (@keyList){
		if ($key<=$keyLimit){
			delete $currentKeys->{$key};
		}
	}
		
}

sub setSuccessor(){
	my $self = shift;
	my $newSuccessor = shift;
	
	$self->{"successor"} = $newSuccessor; 
}

sub setPredecessor(){
	my $self = shift;
	my $newPredecessor = shift;
	
	$self->{"predecessor"} = $newPredecessor; 
}

sub setKeys(){
	my $self = shift;
	my $newKeys = shift;
	
	$self->{"keys"} = $newKeys;
	
}

sub setIP(){
	my $self=shift;
	my ($ip) = @_;
	
	$self->{"ip"} = $ip;
}

#it sets the finger table
sub setFingerTable(){
	my $self = shift;
	my $fingerTable;
	
	$self->{"fingerTable"} = $fingerTable;
}

sub addKey(){
	my $self = shift;
	my $newKey = shift;
	
	#the value 1 means that the prefix is associated to this node. It should ease searching among many keys
	$self->{"keys"}->{$newKey} = 1;
}

#return 1
1