package CoreSim::Chord::FingerTable;

use strict;
#use CoreSim::Chord::ChordNode;

sub new(){
	
	my $invocant = shift;
	my ($chordID,$fingerTableSize, $maxNodesInChordRing) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	my $self = {
		"chordID" => $chordID,
		"fingerTableSize" => $fingerTableSize,
		"maxNodesInChordRing" => $maxNodesInChordRing,
	};
	
	&initializeFingerTable($self, $chordID);
	
	bless($self, $class);
	
	return $self;
}

sub addFinger(){
	my $self = shift;
	my ($start, $interval, $node, $position) = @_;
	
	$self->{"nodes"}->[$position] = &createFinger($self, $start, $interval, $node, $position); 	
	
}

sub initializeFingerTable(){
	my $self = shift;
	my ($chordID) = @_;
	
	my $fingerTableSize = $self->{"fingerTableSize"};
	
	my $maxNodesInChordRing = $self->{"maxNodesInChordRing"}; 

	for (my $i=0; $i< $fingerTableSize; $i++){
		my $start = ( $chordID + 2**$i ) % $maxNodesInChordRing;
		my $next =  ( $chordID + 2**($i+1) ) % $maxNodesInChordRing;
		
		$self->{"nodes"}->[$i]->{"start"} = $start;
		$self->{"nodes"}->[$i]->{"interval"} = [$start, $next ];
	}	
}

sub createFinger(){
	my $self = shift;
	my ($start, $interval, $node, $position) = @_;
	
	my $finger = {};
	$finger->{"start"} = $start;
	$finger->{"interval"} = $interval;
	$finger->{"node"} = $node;
	
	return $finger;
}

#get the "node" value for a finger
sub getNodeForFinger(){
	my $self = shift;
	my $position = shift;
	
#	printf("The requested position = %s for the node %s and a random request=%s \n", $position, $self->{"chordID"}, $self->{"nodes"}->[1]->{"node"} );
#	printf("this will return=%s. The position = %s\n",$self->{"nodes"}->[$position]->{"node"}->getPOP(), $position);
	return $self->{"nodes"}->[$position]->{"node"};
}

#set the "node" value for a finger
sub setNodeForFinger(){
	my $self = shift;
	my ($position, $newNode) = @_;

#	printf("we're adding %s on the position %s and the node is %d!\n", $newNode->getPOP(),$position, $self->{"chordID"});
	$self->{"nodes"}->[$position]->{"node"} = $newNode;
}

#get the "interval" value for a finger
sub getIntervalForFinger(){
	my $self = shift;
	my $position = shift;
	
	return $self->{"nodes"}->[$position]->{"interval"};
}

sub getFingerTableSize(){
	my $self = shift;
	
	#need to check this
	return scalar( @{ $self->{"nodes"} } );
}


#return 1 at the end
1
