package CoreSim::ITR::InFlightBuffer;

$|=0;
use strict;
use CoreSim::Utilities::Timeline;

sub new(){
	my $invocant = shift;
	
	#the max number of packets is not used for now
	my ($mapCache, $packetBuffer, $ipUtility, $outputPath, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	my %inFlightBuffer; 
	my $timeline = CoreSim::Utilities::Timeline->new($ipUtility, $_DEBUG);
	my $outputFile = "$outputPath/inFlightBuffer.txt";
	my $requestsInBuffer = 0;
	
	my $self={
		"inFlightBuffer" =>\%inFlightBuffer,
		"mapCache" => $mapCache,
		"packetBuffer" => $packetBuffer,
		"ipUtility" => $ipUtility,
		"requestsInBuffer" => $requestsInBuffer,
		"outputFile" => $outputFile,
		"timeline" => $timeline,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
	
	&createOutputFile($self);
	
	return $self;
}

sub addEntry(){
	my $self=shift;
	my ($destinationIP, $expireTimestamp, $packet, $destinationPrefix) = @_;
	
	printf("(IFB) Adding a new entry into inFlightBuffer for IP %s\n", $packet->getDestinationIP()) if ($self->{"_DEBUG"}>=2);
	#destinationInformation can be the destinationPrefix or the destinationIP
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	my $timeline = $self->{"timeline"};
	
	my %bufferElement;
	$bufferElement{"destinationIP"} = $destinationIP;
	$bufferElement{"expireTimestamp"} = $expireTimestamp;
	$bufferElement{"packet"} = $packet;
	$bufferElement{"destinationPrefix"} = $destinationPrefix;
	
	#the key should be unique, that is why before introducing an element in the inFlightBuffer a check that the destinationIP
	#already exists must be made
	$inFlightBuffer->{$destinationIP} = \%bufferElement;
	
	$timeline->addElement($expireTimestamp->getTimestampString(), \%bufferElement);
	
	#increment counter
	$self->{"requestsInBuffer"} += 1;
	
	#write to outputFile
	my $action = 1;
	my $requestsInBuffer = $self->{"requestsInBuffer"};
	my $timestamp = $packet->getTimestamp();
	
	printf("(IFB) adding %s %s %s to inFlightBuffer\n", $timestamp, $destinationIP, $requestsInBuffer) if ($self->{"_DEBUG"}>=3);
	printf("(IFB) The expiretimestamp of this request is %s\n", $expireTimestamp->getTimestampString()) if ($self->{"_DEBUG"}>=3);
	&writeToOutputFile($self, $action, $timestamp, $destinationPrefix, $destinationIP, $requestsInBuffer);
	
}

#old update , before using the timeline
sub update(){
	my $self=shift;
	my ($currentTimestamp) = @_;
	
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	my $mapCache = $self->{"mapCache"};
	my $packetBuffer = $self->{"packetBuffer"};
	
	my @bufferKeys = keys %$inFlightBuffer;
	
	foreach my $key (@bufferKeys){
		my $expireTimestamp = $inFlightBuffer->{$key}->{"expireTimestamp"};
		
		#if the entry has expired we need to put it in the prefix cache
		if ( $currentTimestamp->greaterThan($expireTimestamp) ){
			
			my $destinationIP = $inFlightBuffer->{$key}->{"destinationIP"};
			my $destinationPrefix = $inFlightBuffer->{$key}->{"destinationPrefix"};
			
			#expireTimestamp from InFlight buffer is the entry timestamp for mapCache
			$mapCache->addCacheEntry($destinationPrefix, $expireTimestamp);
			
			#delete the packets from packetBuffer which have the destinationIP within the newly cached prefix(destinationInformation)
			#we need to provide here the destinationPrefix! 
			$packetBuffer->purgeSolved($destinationPrefix);
			
			$self->{"requestsInBuffer"} -= 1;
			
			my $packet = $inFlightBuffer->{$key}->{"packet"};
			
			#write to outputFile
			my $action = -1;
			my $requestsInBuffer = $self->{"requestsInBuffer"};
			my $timestamp = $expireTimestamp->getTimestampString();
			
			&writeToOutputFile($self, $action, $timestamp, $destinationPrefix, $destinationIP, $requestsInBuffer);
			
			#delete the entry from inFlight buffer
			delete $inFlightBuffer->{$key};
		}
	}
	
}

sub update1(){
	my $self=shift;
	my ($currentTimestamp) = @_;
	
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	my $mapCache = $self->{"mapCache"};
	my $packetBuffer = $self->{"packetBuffer"};	
	my $timeline = $self->{"timeline"};
	
	printf("(IFB) Starting the update of the inFlightBuffer\n") if ($self->{"_DEBUG"}>=3);
	my $endLoop=0;
	
	#delete all the "old" entries
	while(!$endLoop){
		
		#retrieves the object from the timeline
		my $firstElement = $timeline->firstElement();
		
		if(defined($firstElement)){
			
			my $expireTimestamp = $firstElement->{"expireTimestamp"};
			
			printf("(IFB) The expire timestamp of the first element %s is %s. Current timestamp is %s \n",$firstElement->{"destinationIP"}, 
			$expireTimestamp->getTimestampString(), $currentTimestamp->getTimestampString() ) if ($self->{"_DEBUG"}>=3);

			
			if ( $currentTimestamp->greaterThan( $expireTimestamp ) ){
				
				my $destinationIP = $firstElement->{"destinationIP"};
				my $destinationPrefix = $firstElement->{"destinationPrefix"};
				printf("(IFB) Deleting from inFlightBuffer the request for %s\n", $destinationPrefix) if ($self->{"_DEBUG"}>=3);
				
				
				#expireTimestamp from InFlight buffer is the entry timestamp for mapCache
				$mapCache->addCacheEntry($destinationPrefix, $expireTimestamp);
				
				#delete the packets from packetBuffer which have the destinationIP within the newly cached prefix(destinationInformation)
				#we need to provide here the destinationPrefix! 
				$packetBuffer->purgeSolved($destinationPrefix);
				
				$self->{"requestsInBuffer"} -= 1;
				
				my $packet = $firstElement->{"packet"};
				
				#write to outputFile
				my $action = -1;
				my $requestsInBuffer = $self->{"requestsInBuffer"};
				my $timestamp = $expireTimestamp->getTimestampString();
				
				&writeToOutputFile($self, $action, $timestamp, $destinationPrefix, $destinationIP, $requestsInBuffer);
				
				#delete the entry in the inFlightBuffer for the destinationIP
				delete $inFlightBuffer->{$destinationIP};
			
				#delete the firstElement from the timeline
				$timeline->deleteFirstElement();
				
				printf("(IFB) We still have %s entries in the inFlightBuffer\n", scalar(keys %$inFlightBuffer)) if ($self->{"_DEBUG"}>=3);
			#we finished the elements with the timestamp lower than the current timestamp
			}else{
				$endLoop=1;
			}
		
		#no more elements in the timeline
		}else{
			$endLoop=1;
		}
		
		
	}
	
	
}

sub checkExistingMapRequest(){
	my $self=shift;
	my ($destinationInformation) = @_;
	
	printf("(IFB) Request to check if a map request exists for IP %s\n", $destinationInformation) if ($self->{"_DEBUG"});
	#destinationInformation may be destinationIP or destinationPrefix
	
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	
	if ( defined($inFlightBuffer->{$destinationInformation}) ){
		printf("(IFB) A map request for %s exists!\n", $destinationInformation) if ($self->{"_DEBUG"});
		return 1;
	}else{
		return 0;
	}
}

sub getPacketForMapRequest(){
	my $self=shift;
	my ($destinationInformation) = @_;
	
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	
	return $inFlightBuffer->{$destinationInformation}->{"packet"};
}

sub getExpireTimestampForMapRequest(){
	my $self=shift;
	
	my ($destinationInformation) = @_;
	
	my $inFlightBuffer = $self->{"inFlightBuffer"};
	
	return $inFlightBuffer->{$destinationInformation}->{"expireTimestamp"};	
}

sub createOutputFile(){
	my $self=shift;
	
	my $outputFile = $self->{"outputFile"};
	
	open(OFILE, "> $outputFile") or die printf("Error opening the file %s associated to the ITR!", $outputFile);
	
	
	printf(OFILE "action\ttimestamp\tdestination prefix\tdestination IP\trequestsInBuffer\n");
	
	close(OFILE);
	
}

sub writeToOutputFile(){
	my ($self, $action, $timestamp, $prefix, $destIP, $requestsInBuffer) = @_;
	
	my $outputFile = $self->{"outputFile"};
	
	open(OFILE, ">> $outputFile") or die printf("Error while opening the output file for mapCache %s\n:%s", $outputFile, $!);
	
	#add value to cache
	if ($action == 1){
		printf(OFILE "add\t%s\t%s\t%s\t%s\n", $timestamp, $prefix,$destIP, $requestsInBuffer);
	}elsif($action == -1){
		printf(OFILE "del\t%s\t%s\t%s\t%s\n", $timestamp, $prefix, $destIP, $requestsInBuffer);
	}
	
	close(OFILE);
}


#return 1 at the end
1