package CoreSim::ITR::PacketBuffer;

use strict;

sub new(){
	my $invocant = shift;
	
	my ( $ipUtility, $outputPath, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	my %packetBuffer=();
	my $outputFile = "$outputPath/packetBuffer.txt";
	
	my $nbOfPacketsInBuffer =0;
	my $bytesInBuffer = 0;
	my $nbTCPPackets = 0;
	my $nbUDPPackets = 0;
	my $tcpBytes = 0;
	my $udpBytes = 0;
	
	
	
	my $self={ 
		"packetBuffer" =>\%packetBuffer,
		"ipUtility" => $ipUtility,
		"outputFile" => $outputFile,
		"nbOfPacketsInBuffer" => $nbOfPacketsInBuffer,
		"bytesInBuffer" => $bytesInBuffer,
		"nbTCPPackets" => $nbTCPPackets,
		"nbUDPPackets" => $nbUDPPackets,
		"tcpBytes" => $tcpBytes,
		"udpBytes" => $udpBytes,
		"_DEBUG" => $_DEBUG,
		
	};
	
	bless($self, $class);
	
	&createOutputFile($self);
	
	return $self;
}

sub addPacket(){
	my $self=shift;
	my ($packet, $expireTimestamp) = @_;
	
	my $packetBuffer = $self->{"packetBuffer"};
	
	#create a hash which contains the packet ant the expireTimestamp
	my %bufferElement;
	$bufferElement{"packet"} = $packet;
	$bufferElement{"expireTimestamp"} = $expireTimestamp;
	
	my $entryTimestamp = $packet->getTimestamp();
	my $destinationIP = $packet->getDestinationIP();
	
	#push the new hash into the list associated to each destinationIP
	push ( @{ $packetBuffer->{$destinationIP} } , \%bufferElement ); 
	
	my $action = 1;
	&updateVariables($self, $action, $packet, $entryTimestamp);
		
}

sub update(){
	my $self = shift;
	my ($currentTimestamp) = @_;
	
	my $packetBuffer = $self->{"packetBuffer"};
	
	my @bufferKeys = keys %$packetBuffer;
	
	#take all the packets and eliminate those which have the expireTime smaller than the current timestamp
	foreach my $element (@bufferKeys){
		my $expireTimestamp = $packetBuffer->{$element}->{"expireTimestamp"};
		
		if ( $currentTimestamp->greaterThan($expireTimestamp) ){
			
			my $packet = $packetBuffer->{$element}->{"packet"};
			
			#action=-1 means delete
			my $action = -1;
			my $eTimestamp = $expireTimestamp->getTimestampString();
			&updateVariables($self, $action, $packet, $eTimestamp);
			
			#delete the packet
			delete $packetBuffer->{$element};
			
		}
		
	}
	
}

sub numberOfPacketsInBuffer(){
	my $self=shift;
	
	return $self->{"nbOfPacketsInBuffer"};
}

#it deletes all the packets which have their IPs whithin the prefix given as argument
#this is called when a MAP-Reply is received
sub purgeSolved(){ 
	my $self=shift;
	my ($destinationPrefix) = @_;
	
	my $ipUtility = $self->{"ipUtility"};
	my $packetBuffer = $self->{"packetBuffer"};
	
	my @destinationIPs = keys %$packetBuffer;
	
	#for all the packets in the buffer check to see if their destination IP is within the destinationPrefix
	foreach my $destinationIP (@destinationIPs){
#		my $packetIP = $packetBuffer->{$bufferElement}->{"packet"}->getDestinationIP();
		
		#if the destinationIP is inside the destinationPrefix
		if ($ipUtility->ipInsidePrefix($destinationIP, $destinationPrefix)){
			
			#need to eliminate all the packets associated to this IP
			my $packetList = $packetBuffer->{$destinationIP};
			
			printf("(PB) %s packets associated to destinationIP %s are to be deleted in purge solved!\n", 
					scalar(@$packetList), $destinationIP) if ($self->{"_DEBUG"}>=2);
				
			foreach my $packetItem (@$packetList){
				
				my $packet = $packetItem->{"packet"};
				my $action = -1;
				my $eTimestamp = $packetItem->{"expireTimestamp"}->getTimestampString();
				&updateVariables($self, $action, $packet, $eTimestamp);
			}
			
			#delete the whole entry in the hash
			delete $packetBuffer->{$destinationIP};
		}

	}
}

sub updateVariables(){
	my ($self, $action, $packet, $timestamp) = @_;
	
	#add
	if ($action ==1 ){
		
		#update variables
		$self->{"nbOfPacketsInBuffer"} += 1; 
		$self->{"bytesInBuffer"} += $packet->getSize();
		
		if ($packet->getProtocol() == 6){
			$self->{"nbTCPPackets"} += 1;
			$self->{"tcpBytes"} += $packet->getSize();
		}elsif( $packet->getProtocol() == 17){
			$self->{"nbUDPPackets"} += 1;
			$self->{"udpBytes"} += $packet->getSize();
		}
		
		
		#write to outputfile
		my $destIP = $packet->getDestinationIP();
		my $protocol = $packet->getProtocol();
		my $size = $packet->getSize();
		my $nbOfPacketsInBuffer = $self->{"nbOfPacketsInBuffer"}; 
		my $bytesInBuffer = $self->{"bytesInBuffer"};
		my $nbTCPPackets = $self->{"nbTCPPackets"};
		my $nbUDPPackets = $self->{"nbUDPPackets"};
		my $tcpBytes = $self->{"tcpBytes"};
		my $udpBytes = $self->{"udpBytes"};
		
		&writeToOutputFile($self,$action, $destIP, $timestamp, $protocol, $size, $nbOfPacketsInBuffer, $bytesInBuffer, $nbTCPPackets, $nbUDPPackets, 
						   $tcpBytes, $udpBytes);
						   
		
	#delete
	}elsif($action ==-1){
		
		#update the variables
		$self->{"nbOfPacketsInBuffer"} -= 1; 
		$self->{"bytesInBuffer"} -= $packet->getSize();
		 
		if ($packet->getProtocol() == 6){
			$self->{"nbTCPPackets"} -= 1;
			$self->{"tcpBytes"} -= $packet->getSize();
		}elsif( $packet->getProtocol() == 17){
			$self->{"nbUDPPackets"} -= 1;
			$self->{"udpBytes"} -= $packet->getSize();
		}
		
		#write to outputfile
		my $destIP = $packet->getDestinationIP();
		my $protocol = $packet->getProtocol();
		my $size = $packet->getSize();
		my $nbOfPacketsInBuffer = $self->{"nbOfPacketsInBuffer"}; 
		my $bytesInBuffer = $self->{"bytesInBuffer"};
		my $nbTCPPackets = $self->{"nbTCPPackets"};
		my $nbUDPPackets = $self->{"nbUDPPackets"};
		my $tcpBytes = $self->{"tcpBytes"};
		my $udpBytes = $self->{"udpBytes"};
		
		&writeToOutputFile($self, $action, $destIP, $timestamp, $protocol, $size, $nbOfPacketsInBuffer, $bytesInBuffer, $nbTCPPackets, $nbUDPPackets, 
						   $tcpBytes, $udpBytes);
			
		
	}
}

sub createOutputFile(){
	my $self=shift;
	
	my $outputFile = $self->{"outputFile"};
	
	open(OFILE, "> $outputFile") or die printf("Error opening the file %s associated to the ITR!", $outputFile);
	
	my $timeout = $self->{"timeout"};
	my $maxNbOfEntriesInCache = $self->{"maxNbOfEntriesInCache"};
	
	printf(OFILE "action\tdestination IP\ttimestamp\tprotocol\tsize\tnbOfPackets\tbytesInBuffer\tnbTCPPackets\tnbUDPPackets\ttcpBytes\tudpBytes\n");
	
	$self->{"packetBufferFileHandle"} = \*OFILE;
	
}

sub writeToOutputFile(){
	my ($self,$action, $destIP, $timestamp, $protocol, $size, $nbOfPacketsInBuffer, $bytesInBuffer, $nbTCPPackets, $nbUDPPackets, $tcpBytes, $udpBytes) = @_;
	
	my $packetBufferFileHandle = $self->{"packetBufferFileHandle"};
		
	#add value to cache
	if ($action == 1){
		printf($packetBufferFileHandle "add\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", $destIP, $timestamp, $protocol, $size, $nbOfPacketsInBuffer, $bytesInBuffer, $nbTCPPackets, $nbUDPPackets, $tcpBytes, $udpBytes);
	}elsif($action == -1){
		printf($packetBufferFileHandle "del\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", $destIP, $timestamp, $protocol, $size, $nbOfPacketsInBuffer, $bytesInBuffer, $nbTCPPackets, $nbUDPPackets, $tcpBytes, $udpBytes);
	}
	
}

sub DESTROY(){
	my $self=shift;
	
	printf("(PB) Closing packet buffer file handle\n");
	
	my $packetBufferFileHandle = $self->{"packetBufferFileHandle"};
	close($packetBufferFileHandle);
}
#return 1 at the end
1