package CoreSim::Utilities::IPUtilities;

use strict;

sub new(){
	my $invocant = shift;
	
	my $class = ref($invocant) || $invocant;
	
	my $self={};
	
	bless($self, $class);
	
	return $self;
		
}

sub ipToNumber(){
	
	my $ip = shift;
	
	if (defined($ip)){
		my @data = split(/\./, $ip);
		
		if (!defined($data[0]) || !defined($data[1]) || !defined($data[2]) || !defined($data[3]) ){
			return undef;
		}else{
		
			my $number = $data[0] * (2**24) + $data[1] * (2**16) + $data[2] * (2**8) + $data[3];
			return $number;
		}
	}
	
}

sub maskToNumber(){

        my $mask = shift;

#        my $maskString = "";
#
#        for (my $i=0;$i<32;$i++){
#                if ($i< $mask){
#                        $maskString .="1";
#                }else{
#                        $maskString .="0";
#                }
#        }
#
#        return oct("0b$maskString");
		my $maskToNumber =  ( (2**$mask) -1 ) << (32 - $mask);
		return $maskToNumber
}

sub andIPWithMask(){
        my $ip = shift;
        my $mask = shift;
        my $ipNumber = &ipToNumber($ip);
        my $maskToNumber = &maskToNumber($mask);

#       printf("ip = %s, mask = %s\n",$ipNumber, $maskToNumber);
        my $andedIP = $ipNumber & $maskToNumber;
#        printf("anded = %s\n", $andedIP);
        return &numberToIP(undef, $andedIP);   
}

sub computeChordID(){
	my $prefix = shift;
	if (defined($prefix)){
		
		my @data = split(/\//, $prefix);
		my $networkAddress = $data[0];
		my $mask = $data[1];
		
		my $ipToNumber = &ipToNumber($networkAddress);
		my $nonMaskedToNumber = &getNonmaskedToNumber(undef, $mask);
		
		my $chordID = $ipToNumber + $nonMaskedToNumber;
		return $chordID;
		
	}else{
		return undef;
	}
		
}

sub prefixToNumber(){
	my $prefix = shift;
	
	my @data = split(/\//, $prefix);
	my $networkAddress = $data[0];
	
	my $number = &ipToNumber($networkAddress); 
}

#functions called from the "outside"

sub numberToIP(){
	my $self = shift;
	my $number = shift;
	
#	my $numberBinary = sprintf("%b", $number);
#	my $numberString = "$numberBinary";
#	
#	#prepend with 0 until there are 32 bits
#	for (my $i= length($numberString); $i< 32; $i++){
#		$numberString = "0".$numberString;
#	}
#
#	my $ip;
#	for (my $i=0; $i< 4; $i++){
#		my $subNum = substr($numberString, $i*8, 8);
#		
#		if ($i!=3){
#			$ip .= oct("0b$subNum").".";
#		}else{
#			$ip .= oct("0b$subNum");
#		}
#	}
	
	my $power=24;
	my $tmpNumber = $number;
	my $ip;
	
	for (my $i=0; $i<4; $i++){
		my $part = int( $tmpNumber/ (2**$power) );
		$tmpNumber =  $tmpNumber % (2**$power);
		
		if ($i!=3){
			$ip .="$part.";
		}else{
			$ip .="$part";
		}
		
		$power = $power - 8;
	}
	return $ip;
	
}

sub getIPToNumber(){
	my $self=shift;
	my $ip = shift;
	
	return &ipToNumber($ip);
}

sub computeParentPrefix(){
	my $self=shift;
	my ($prefix, $nbOfAggregatedBits) = @_;	
	
	my $currentMask = &getMaskFromPrefix($self, $prefix);
	my $currentIP = &getIPFromPrefix($self, $prefix);
	
	my $newMask = $currentMask - $nbOfAggregatedBits;
	
	my $newIP = &andIPWithMask($currentIP, $newMask);
	
	my $newPrefix = "$newIP/$newMask";
#	printf("the new prefix = %s\n", $newPrefix);
	
	return $newPrefix;
	
}

#return a number computed from the bits which are not masked with value 1
sub getNonmaskedToNumber(){
	my $self = shift;
	my $mask = shift;
	
	if (defined($mask)){
		return 2**(32-$mask)-1;
	}else{
		return undef;
	}
}

sub getBroadcastNumberFromPrefix(){
	my $self = shift;
	my $prefix = shift;
	
	return &computeChordID($prefix);
}

sub getNetworkNumberFromPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	my @list = split(/\//, $prefix);
	return &ipToNumber($list[0]);	
}

sub getChordIDForPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	return &computeChordID($prefix);	
}

sub getIPFromPrefix(){
	my ($self,$prefix) = @_;
	
	my @result = split(/\//, $prefix);
	return $result[0];	
}

#returns the part after / from prefix
sub getMaskFromPrefix(){
	my $self=shift;
	my $string = shift;
	
	my @result = split(/\//, $string);
	return $result[1];	
}

sub getPrefixToNumber(){
	my $self = shift;
	my $prefix = shift;
	
	return &prefixToNumber($prefix);
}

sub ipInsidePrefix(){
	my $self=shift;
	my ($ip, $prefix) = @_;
	
	my $ipToNumber = &ipToNumber($ip);
	
	if ( (&getNetworkNumberFromPrefix($self, $prefix)<= $ipToNumber ) && ( $ipToNumber <= &getBroadcastNumberFromPrefix($self,$prefix)) ){
		return 1;
	}else{
		return 0;
	}
}

sub prefixInsidePrefix(){
	my $self = shift;
	my ($prefix1, $prefix2) = @_;
	
	if ( ( &getNetworkNumberFromPrefix($self,$prefix2) <= &getNetworkNumberFromPrefix($self,$prefix1)) && 
			(&getBroadcastNumberFromPrefix($self,$prefix1)<=&getBroadcastNumberFromPrefix($self,$prefix2)) ){
		return 1;
	}else{
		return 0;
	}
	
}

#return 1
1