package CoreSim::Utilities::Timestamp;

use strict;

sub new(){
	my $invocant = shift;
	
	my $timestamp = shift;
	
	my $class = ref($invocant) || $invocant;

	my $self={
		"string" => $timestamp,
	};
	
	&parseString($self, $timestamp );
	
	bless($self, $class);
	return $self;
	
}

sub parseString(){
	my $self = shift;
	my $string = shift;
	
	#the format is <32_bit_seconds><32_bit_fractions>
	#the strings will be 16 characters long. i split them into 2 of 8 such that, when transformed to decimal integers from hexadecimal 
	#they will work on 32bit processors also
	
	my $seconds = hex( substr($string,0, 8) );
	my $fraction = hex( substr($string, 8, 8) );
	
	$self->{"seconds"} = $seconds;
	$self->{"fractions"} = $fraction;
	
	
}

sub parseStringDamien(){
	my $self = shift;
	my $string = shift;
	
	#input format is <day_of_year>.<HH:mm:ss>.<milliseconds>
	#since there is no year information, the result will not be in epoch!! but since we're intreseted in differences, that's OK
	
	(my $day, my $time, my $ms) = split(/\./,$string);
    (my $hour, my $min, my $sec) = split(/:/,$time);
    my $seconds = 86400 * $day + 3600 * $hour + 60 * $min + $sec;
    my $fraction = int( (2**32 / 1000.0) * $ms );
	
	$self->{"seconds"} = $seconds;
	$self->{"fractions"} = $fraction;
}

sub subtract(){
	my $self= shift;
	my ($operand2) = @_;
	
	my $fractionsOp1 = $self->{"fractions"};
	my $fractionsOp2 = $operand2->getFractions();
	
	my $secondsOp1 = $self->{"seconds"};
	my $secondsOp2 = $operand2->getSeconds();
	
	if( ($secondsOp1< $secondsOp2)|| ( ( $secondsOp1==$secondsOp2)&&($fractionsOp1< $fractionsOp2) ) ){
		printf("(EE) There is an issue with the timestamps in substract(). The first operand is smaller than the second!\n");
		return undef;
	}
	
	my $fractionsDiff;
	my $secondsDiff;
	
	#we need to borrow from the seconds
	if ($fractionsOp1<$fractionsOp2){
		$fractionsDiff = 2**31-$fractionsOp2+$fractionsOp1;
		$fractionsDiff += 2**31;
		
		#we need to substract 1 more due to the "borrowing"
		$secondsDiff = $secondsOp1 - $secondsOp2 -1;
	
		my %hashResult;
		$hashResult{"seconds"} = $secondsDiff;
		$hashResult{"fractions"} = $fractionsDiff;
		return \%hashResult;
			
	}else{
		$fractionsDiff = $fractionsOp1-$fractionsOp2;
		$secondsDiff = $secondsOp1 - $secondsOp2;
		
		my %hashResult;
		$hashResult{"seconds"} = $secondsDiff;
		$hashResult{"fractions"} = $fractionsDiff;
		return \%hashResult;
	}
	
}

sub add(){
	my $self=shift;
	
	my $operand2 = shift;
	
	my $secondsOp1 = $self->{"seconds"};
	my $secondsOp2 = $operand2->getSeconds();
	
	my $fractionsOp1 = $self->{"fractions"};
	my $fractionsOp2 = $operand2->getFractions();
	
	
}

sub addValue(){
	my $self=shift;
	
	my $msValue = shift;
	
	my $secondsOp1 = $self->{"seconds"};
	my $fractionsOp1 = $self->{"fractions"};
	
	my $msOnly = $msValue % 1000;
	my $sOnly = int($msValue / 1000);
	
	my $msToFractions = int( (2**32 / 1000.0) * $msOnly );
	
	my $carry=0;
	my $newFractions =0;
	
	#the fractions will overflow
	if ($msToFractions + $fractionsOp1 >= 2**32 ){
		
		$carry=1;
		$newFractions = $fractionsOp1 + $msToFractions - 2**32;
		
	#the fractions won't overflow
	}else{
		
		$carry = 0;
		$newFractions = $fractionsOp1 + $msToFractions;
		
	}
	
	my $newSeconds = $secondsOp1+$sOnly + $carry;
	
	my $hexFractions = sprintf("%x", $newFractions);
	my $hexSeconds = sprintf("%x", $newSeconds);
	
	#complete with 0 the fractions if the length of the hexa number is less than 8 characters
	my $stringHexFractions = $hexFractions;
	while (length($stringHexFractions) < 8){
		$stringHexFractions = "0".$stringHexFractions;
	}
	my $hexTimestamp = "$hexSeconds$stringHexFractions";
	
	
	my $newTimestamp = CoreSim::Utilities::Timestamp->new($hexTimestamp);
	
	return $newTimestamp;
	
	
}

#compares current timestamp(operand1) and the timesamp object given as argument(operand2)
#should I compare the hexa values directly?
sub greaterThan(){
	my $self=shift;
	my $operand2 = shift;
	
#	my $secondsOp1 = $self->{"seconds"};
#	my $secondsOp2 = $operand2->getSeconds();
#	
#	my $fractionsOp1 = $self->{"fractions"};
#	my $fractionsOp2 = $operand2->getFractions();
#	
#	if ($secondsOp1>$secondsOp2){
#		return 1;
#	}elsif( ($secondsOp1 == $secondsOp2) && ($fractionsOp1>$fractionsOp2)){
#		return 1;
#	}else{
#		return 0;
#	}
	my $string1 = $self->{"string"};
	my $string2 = $operand2->getTimestampString();
	
	if ($string1 gt $string2){
		return 1;
	}else{
		return 0;
	}
	
}

sub getFractions(){
	my $self=shift;
	return $self->{"fractions"};
}

sub getSeconds(){
	my $self = shift;
	return $self->{"seconds"};
}

sub getTimestampString(){
	my $self = shift;
	return $self->{"string"};
}
#return 1
1